
#Tutorial 2 - Topics: Definition of Functions, Simple Correlation Analysis of Two Signals
# and Iterative Functions

from numpy import arange,sin,pi,mean,std,savetxt,zeros,dot,loadtxt
from matplotlib.pyplot import  show,figure,plot,subplot,title,xlabel,ylabel,grid
from numpy.random import randn
##==============================================================================
## Part 1 - Definition of Functions 
##==============================================================================
t=arange(0,60.01,0.01)
#
def signal1(t):
    y = 20*sin(0.15*t) + randn(1)
    return y

def signal2(t):
    x=10+20*sin(0.15*t)
    return x
#    
#y = signal1(t) # To apply our newly defined functions
#x = signal2(t)
#
    
# The correlation coefficient r(x,y) is essentially a measure
# of the linear dependancy that the signal x has with the signal y
# We may use our new skill of defining function to define the correlation
# function, which may be reused on other signal throughout our program without
# re-writting the function! 
    
def corrcoeff(x,y,meany,stdy,meanx,stdx):
    R = zeros(len(y))
    for k in range(0,len(y)):
        R[k] = mean(((x[k]-meanx)/stdx)*((y[k]-meany)/stdy))
        
    return R
   
y = signal1(t)
x = signal2(t)
N = len(x)
   
meany=mean(y)
stdy=std(y)
meanx=mean(x)
stdx=std(x)

print 'meany =', meany,  'stdy =', stdy,  'meanx =', meanx,  'stdx =', stdx

R = corrcoeff(x,y,meany,stdy,meanx,stdx)

figure()
subplot(311)
plot(y,'r'),grid(),xlabel('time[s]'),ylabel('Amplitude[mm]'),title('Signal 1 = Y')
subplot(312)
plot(x,'b'),grid(),xlabel('time[s]'),ylabel('Amplitude[mm]'),title('Signal 2 = X')
subplot(313)
plot(R,'k'),grid(),xlabel('Samples as time steps'),ylabel('R(x,y)'),title('Correlation Between X & Y')
show()

#==============================================================================
# Part 2 - Iterative Functions
#==============================================================================

# Motivation:
# Often in measurement and control of engineering processes, it is neccesary to 
# read measured values from a sensor (recorded usually in a txt file) and provide
# a further calculation to send new values back to the process (i.e; an actuator)
# for example: application of a PID or PLC controller
 

#If we desire to save our signal into a txt file
#Y = signal1(t)
#savetxt('yout.txt',Y)

#Here we define our iterative function procedure. In this example we just
#want to generate a simple function y = f(w,y[k-i]) for i=0,1,2,3 (dependent of coeff's w)
#First we write an iterative function which updates the newly calculated values of y[k+1]
#Into our initial zero(empty) array

def yout(w,y):
    N = len(y)
    Y = zeros(N)
    for k in range(0,N-1): # 0, N-1 = to preserve the correct range of the data y
        Y[k+1]=w[0]*y[k]+w[1]*y[k-1]+w[2]*y[k-2]+w[-1]*y[k-3] # Difference Equation. May principally represent an iterative
                                                                # model of the process
    return Y
    
# Now we define the vector w (which are coefficients of the iterative function y[k+1])
w = [0.30231469, 0.23231564, 0.1425327, -0.033339]
print w 
#
y = loadtxt('y.txt') # Introduce the measured data from the process
#
Y = yout(w,y) # Return the newly calculated function. It may be new signals to send to an actuator
# or a mathematical model of the process as was inspired in this case.


figure()
subplot(211)
plot(y),grid(),title('Original Function')
subplot(212)
plot(Y,'g'),grid(),title('Iterative Function')
show()





