from numpy import sin, zeros, mean, std, arange
from numpy.random import randn
import signals
from signals import Signals
from matplotlib.pyplot import figure, subplot, plot, grid, title, xlabel, ylabel, show

"""
Using Classes Imported from a Our Own Module

Apart from defining a class in our main script, we may also save it in a 
separate module. This is useful when we are creating a large main code, to keep
structure and clarity in the main file. In this case we have a module named
"signals.py" which features a class "Signals". Here "Signals"
is a template for an Object (or instance of the Class) that features attributes such as 
the input signals "signal1 & 2" and also the correlation coefficient between "signal1 & 2"
the methods/functions of these attributes help us to compute/define these attributes
"""

#class Signals():
#    def __init__(self,name):
#        self.name = name        
#        self.signal1 = [] 
#        self.signal2 = []
#        self.Rxy = []
#        
#    def SinWithNoise(self,t):
#        self.signal1 = 20*sin(0.15*t) + randn(1)
#        return self.signal1
#
#    def SinNormal(self,t):
#        self.signal2 = 10+20*sin(0.15*t)
#        return self.signal2
#    
#    def corrcoeff(self,x,y,meany,stdy,meanx,stdx):
#        R = zeros(len(y))
#        for k in range(0,len(y)):
#            R[k] = mean(((x[k]-meanx)/stdx)*((y[k]-meany)/stdy))
#            self.Rxy.append(R[k])
#        return R

"""
Once our module is imported, we must construct an object (= instance of the class) associated with the class
"Signals". The brackets denote any parameters of the Class. In our case, the name of
the signal (or instance of the class).
"""

signal1 = Signals("x")
signal2 = Signals("y")
#
print signal1.name
print signal2.name

"""
On defining our parameter t as input for defining our signal functions
"SinWithNoise" and "SinNormal", we can further calculate the correlation
coefficient between the two signals. Or the correlation between instances signal 1 or signal 2
with any other signal we wish to define in the "corrcoeff" function of the class
"Signals"
"""

t=arange(0,60.01,0.01)

x = signal1.SinWithNoise(t)
y = signal2.SinNormal(t)

meany = mean(y)
stdy = std(y)
meanx = mean(x)
stdx = std(x)

R = signal1.corrcoeff(x,y,meany,stdy,meanx,stdx)

figure()
subplot(311)
plot(y,'r'),grid(),xlabel('time[s]'),ylabel('Amplitude[mm]'),title('Signal 1 = Y')
subplot(312)
plot(x,'b'),grid(),xlabel('time[s]'),ylabel('Amplitude[mm]'),title('Signal 2 = X')
subplot(313)
plot(R,'k'),grid(),xlabel('Samples as time steps'),ylabel('R(x,y)'),title('Correlation Between X & Y')
show()